@@ -9,12 +9,11 @@ import (
 	"strings"
 
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
-	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/factory"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/kube"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/types"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
 	kapi "k8s.io/api/core/v1"
-	"k8s.io/client-go/tools/cache"
 	"k8s.io/klog/v2"
 	utilnet "k8s.io/utils/net"
 )
@@ -25,141 +24,143 @@ const (
 	defaultOpenFlowCookie = "0xdeff105"
 )
 
-func addService(service *kapi.Service, inport, outport, gwBridge string, nodeIP *net.IPNet) {
-	if !util.ServiceTypeHasNodePort(service) && len(service.Spec.ExternalIPs) == 0 {
-		return
-	}
-
-	for _, svcPort := range service.Spec.Ports {
-		protocol := strings.ToLower(string(svcPort.Protocol))
-		protocol6 := protocol + "6"
-		if util.ServiceTypeHasNodePort(service) {
-			if err := util.ValidatePort(svcPort.Protocol, svcPort.NodePort); err != nil {
-				klog.Errorf("Skipping service add for svc: %s, err: %v", svcPort.Name, err)
-				continue
-			}
-			if config.IPv4Mode {
-				_, stderr, err := util.RunOVSOfctl("add-flow", gwBridge,
-					fmt.Sprintf("priority=100, in_port=%s, %s, tp_dst=%d, actions=%s",
-						inport, protocol, svcPort.NodePort, outport))
-				if err != nil {
-					klog.Errorf("Failed to add openflow flow on %s for nodePort: "+
-						"%d, stderr: %q, error: %v", gwBridge, svcPort.NodePort, stderr, err)
-				}
-			}
-			if config.IPv6Mode {
-				_, stderr, err := util.RunOVSOfctl("add-flow", gwBridge,
-					fmt.Sprintf("priority=100, in_port=%s, %s, tp_dst=%d, actions=%s",
-						inport, protocol6, svcPort.NodePort, outport))
-				if err != nil {
-					klog.Errorf("Failed to add openflow flow on %s for nodePort: "+
-						"%d, stderr: %q, error: %v", gwBridge, svcPort.NodePort, stderr, err)
-				}
-			}
-			// Flows for cloud load balancers on Azure/GCP
-			flows := make([]string, 0)
-
-			// Established traffic is handled by default conntrack rules
-			// Table 1 handles forwarding traffic to pods
-			// Table 2 handles return NAT traffic for node port and forwards out of the host
-			// NodePort/Ingress access in the OVS bridge will only ever come from outside of the host
-			for _, ing := range service.Status.LoadBalancer.Ingress {
-				if ing.IP == "" {
-					continue
-				}
-				ingIP := net.ParseIP(ing.IP)
-				if ingIP == nil {
-					klog.Errorf("Failed to parse ingress IP: %s", ing.IP)
-					continue
-				}
-				cookie, err := svcToCookie(service.Name, ingIP, svcPort.Port)
-				if err != nil {
-					klog.Errorf("Unable to generate cookie for svc: %s, %s, %d, error: %v",
-						service.Name, ingIP.String(), svcPort.Port, err)
-					continue
-				}
-				flowProtocol := protocol
-				nw_dst := "nw_dst"
-				nw_src := "nw_src"
-				if utilnet.IsIPv6String(ing.IP) {
-					flowProtocol = protocol6
-					nw_dst = "ipv6_dst"
-					nw_src = "ipv6_src"
-				}
-				// Send to CT from external net with DNAT, eventually will land in table 1 (traffic to pods)
-				flows = append(flows,
-					fmt.Sprintf("cookie=%s,priority=101,in_port=%s,ct_state=-trk,%s,%s=%s,tp_dst=%d,"+
-						"actions=ct(commit,table=0,zone=%d,nat(dst=%s)",
-						cookie, inport, flowProtocol, nw_dst, ing.IP, svcPort.Port, config.Default.ConntrackZone,
-						util.JoinHostPortInt32(nodeIP.IP.String(), svcPort.NodePort)))
-				// Incoming return traffic from pods, send to CT, unDNAT, goto table 2
-				flows = append(flows,
-					fmt.Sprintf("cookie=%s,priority=101,in_port=%s,%s,%s=%s,tp_src=%d,"+
-						"actions=ct(table=2,zone=%d,nat)",
-						cookie, outport, flowProtocol, nw_src, nodeIP.IP, svcPort.NodePort, config.Default.ConntrackZone))
-			}
-
-			for _, flow := range flows {
-				_, stderr, err := util.RunOVSOfctl("add-flow", gwBridge, flow)
-				if err != nil {
-					klog.Errorf("Failed to add openflow flow on %s for ingress, stderr: %q, error: %v, flow: %s",
-						gwBridge, stderr, err, flow)
-				}
-			}
-		}
-		for _, externalIP := range service.Spec.ExternalIPs {
-			if err := util.ValidatePort(svcPort.Protocol, svcPort.Port); err != nil {
-				klog.Errorf("Skipping service add for svc: %s, err: %v", svcPort.Name, err)
-				continue
-			}
-			flowProtocol := protocol
-			nw_dst := "nw_dst"
-			if utilnet.IsIPv6String(externalIP) {
-				flowProtocol = protocol6
-				nw_dst = "ipv6_dst"
-			}
-			_, stderr, err := util.RunOVSOfctl("add-flow", gwBridge,
-				fmt.Sprintf("priority=100, in_port=%s, %s, %s=%s, tp_dst=%d, actions=%s",
-					inport, flowProtocol, nw_dst, externalIP, svcPort.Port, outport))
-			if err != nil {
-				klog.Errorf("Failed to add openflow flow on %s for ExternalIP: "+
-					"%s, stderr: %q, error: %v", gwBridge, externalIP, stderr, err)
-			}
-		}
-	}
-
-	addSharedGatewayIptRules(service, nodeIP)
+// nodePortWatcher manages OpenfLow and iptables rules
+// to ensure that services using NodePorts are accessible
+type nodePortWatcher struct {
+	ofportPhys  string
+	ofportPatch string
+	gwBridge    string
+	nodeIP      *net.IPNet
 }
 
-func deleteService(service *kapi.Service, inport, gwBridge string, nodeIP *net.IPNet) {
-	if !util.ServiceTypeHasNodePort(service) && len(service.Spec.ExternalIPs) == 0 {
+// AddService handles configuring shared gateway bridge flows to steer External IP, Node Port, Ingress LB traffic into OVN
+func (npw *nodePortWatcher) AddService(service *kapi.Service) {
+	// don't process headless service or services that doesn't have NodePorts or ExternalIPs
+	if !util.ServiceTypeHasClusterIP(service) || !util.IsClusterIPSet(service) {
 		return
 	}
-
 	for _, svcPort := range service.Spec.Ports {
 		protocol := strings.ToLower(string(svcPort.Protocol))
 		protocol6 := protocol + "6"
-		if util.ServiceTypeHasNodePort(service) {
-			if err := util.ValidatePort(svcPort.Protocol, svcPort.NodePort); err != nil {
-				klog.Errorf("Skipping service delete, for svc: %s, err: %v", svcPort.Name, err)
-				continue
-			}
+		if svcPort.NodePort > 0 {
 			if config.IPv4Mode {
-				_, stderr, err := util.RunOVSOfctl("del-flows", gwBridge,
-					fmt.Sprintf("in_port=%s, %s, tp_dst=%d",
-						inport, protocol, svcPort.NodePort))
+				_, stderr, err := util.RunOVSOfctl("add-flow", npw.gwBridge,
+					fmt.Sprintf("priority=100, in_port=%s, %s, tp_dst=%d, actions=%s",
+						npw.ofportPhys, protocol, svcPort.NodePort, npw.ofportPatch))
 				if err != nil {
-					klog.Errorf("Failed to delete openflow flow on %s for nodePort: "+
-						"%d, stderr: %q, error: %v", gwBridge, svcPort.NodePort, stderr, err)
+					klog.Errorf("Failed to add openflow flow on %s for nodePort: "+
+						"%d, stderr: %q, error: %v", npw.gwBridge, svcPort.NodePort, stderr, err)
 				}
 			}
 			if config.IPv6Mode {
-				_, stderr, err := util.RunOVSOfctl("del-flows", gwBridge,
+				_, stderr, err := util.RunOVSOfctl("add-flow", npw.gwBridge,
+					fmt.Sprintf("priority=100, in_port=%s, %s, tp_dst=%d, actions=%s",
+						npw.ofportPhys, protocol6, svcPort.NodePort, npw.ofportPatch))
+				if err != nil {
+					klog.Errorf("Failed to add openflow flow on %s for nodePort: "+
+						"%d, stderr: %q, error: %v", npw.gwBridge, svcPort.NodePort, stderr, err)
+				}
+			}
+		}
+		// Flows for cloud load balancers on Azure/GCP
+		flows := make([]string, 0)
+
+		// Established traffic is handled by default conntrack rules
+		// NodePort/Ingress access in the OVS bridge will only ever come from outside of the host
+		for _, ing := range service.Status.LoadBalancer.Ingress {
+			if ing.IP == "" {
+				continue
+			}
+			ingIP := net.ParseIP(ing.IP)
+			if ingIP == nil {
+				klog.Errorf("Failed to parse ingress IP: %s", ing.IP)
+				continue
+			}
+			cookie, err := svcToCookie(service.Name, ingIP, svcPort.Port)
+			if err != nil {
+				klog.Errorf("Unable to generate cookie for svc: %s, %s, %d, error: %v",
+					service.Name, ingIP.String(), svcPort.Port, err)
+				continue
+			}
+			flowProtocol := protocol
+			nwDst := "nw_dst"
+			if utilnet.IsIPv6String(ing.IP) {
+				flowProtocol = protocol6
+				nwDst = "ipv6_dst"
+			}
+			// Send directly to OVN. OVN GR will have load balancer for Ingress IP Service
+			flows = append(flows,
+				fmt.Sprintf("cookie=%s,priority=100,in_port=%s,%s,%s=%s,tp_dst=%d, actions=%s",
+					cookie, npw.ofportPhys, flowProtocol, nwDst, ing.IP, svcPort.Port, npw.ofportPatch))
+		}
+
+		for _, flow := range flows {
+			_, stderr, err := util.RunOVSOfctl("add-flow", npw.gwBridge, flow)
+			if err != nil {
+				klog.Errorf("Failed to add openflow flow on %s for ingress, stderr: %q, error: %v, flow: %s",
+					npw.gwBridge, stderr, err, flow)
+			}
+		}
+
+		for _, externalIP := range service.Spec.ExternalIPs {
+			flowProtocol := protocol
+			nwDst := "nw_dst"
+			if utilnet.IsIPv6String(externalIP) {
+				flowProtocol = protocol6
+				nwDst = "ipv6_dst"
+			}
+			_, stderr, err := util.RunOVSOfctl("add-flow", npw.gwBridge,
+				fmt.Sprintf("priority=100, in_port=%s, %s, %s=%s, tp_dst=%d, actions=%s",
+					npw.ofportPhys, flowProtocol, nwDst, externalIP, svcPort.Port, npw.ofportPatch))
+			if err != nil {
+				klog.Errorf("Failed to add openflow flow on %s for ExternalIP: "+
+					"%s, stderr: %q, error: %v", npw.gwBridge, externalIP, stderr, err)
+			}
+		}
+	}
+
+	addSharedGatewayIptRules(service, npw.nodeIP)
+}
+
+func (npw *nodePortWatcher) UpdateService(old, new *kapi.Service) {
+	if reflect.DeepEqual(new.Spec.Ports, old.Spec.Ports) &&
+		reflect.DeepEqual(new.Spec.ExternalIPs, old.Spec.ExternalIPs) &&
+		reflect.DeepEqual(new.Spec.ClusterIP, old.Spec.ClusterIP) &&
+		reflect.DeepEqual(new.Spec.Type, old.Spec.Type) &&
+		reflect.DeepEqual(new.Status.LoadBalancer.Ingress, old.Status.LoadBalancer.Ingress) {
+		klog.V(5).Infof("Skipping service update for: %s as change does not apply to any of .Spec.Ports, "+
+			".Spec.ExternalIP, .Spec.ClusterIP, .Spec.Type, .Status.LoadBalancer.Ingress", new.Name)
+		return
+	}
+	npw.DeleteService(old)
+	npw.AddService(new)
+}
+
+func (npw *nodePortWatcher) DeleteService(service *kapi.Service) {
+	// don't process headless service
+	if !util.ServiceTypeHasClusterIP(service) || !util.IsClusterIPSet(service) {
+		return
+	}
+	for _, svcPort := range service.Spec.Ports {
+		protocol := strings.ToLower(string(svcPort.Protocol))
+		protocol6 := protocol + "6"
+		if svcPort.NodePort > 0 {
+			if config.IPv4Mode {
+				_, stderr, err := util.RunOVSOfctl("del-flows", npw.gwBridge,
 					fmt.Sprintf("in_port=%s, %s, tp_dst=%d",
-						inport, protocol6, svcPort.NodePort))
+						npw.ofportPhys, protocol, svcPort.NodePort))
 				if err != nil {
 					klog.Errorf("Failed to delete openflow flow on %s for nodePort: "+
-						"%d, stderr: %q, error: %v", gwBridge, svcPort.NodePort, stderr, err)
+						"%d, stderr: %q, error: %v", npw.gwBridge, svcPort.NodePort, stderr, err)
+				}
+			}
+			if config.IPv6Mode {
+				_, stderr, err := util.RunOVSOfctl("del-flows", npw.gwBridge,
+					fmt.Sprintf("in_port=%s, %s, tp_dst=%d",
+						npw.ofportPhys, protocol6, svcPort.NodePort))
+				if err != nil {
+					klog.Errorf("Failed to delete openflow flow on %s for nodePort: "+
+						"%d, stderr: %q, error: %v", npw.gwBridge, svcPort.NodePort, stderr, err)
 				}
 			}
 		}
@@ -174,12 +175,12 @@ func deleteService(service *kapi.Service, inport, gwBridge string, nodeIP *net.I
 				flowProtocol = protocol6
 				nw_dst = "ipv6_dst"
 			}
-			_, stderr, err := util.RunOVSOfctl("del-flows", gwBridge,
+			_, stderr, err := util.RunOVSOfctl("del-flows", npw.gwBridge,
 				fmt.Sprintf("in_port=%s, %s, %s=%s, tp_dst=%d",
-					inport, flowProtocol, nw_dst, externalIP, svcPort.Port))
+					npw.ofportPhys, flowProtocol, nw_dst, externalIP, svcPort.Port))
 			if err != nil {
 				klog.Errorf("Failed to delete openflow flow on %s for ExternalIP: "+
-					"%s, stderr: %q, error: %v", gwBridge, externalIP, stderr, err)
+					"%s, stderr: %q, error: %v", npw.gwBridge, externalIP, stderr, err)
 			}
 		}
 		for _, ing := range service.Status.LoadBalancer.Ingress {
@@ -196,19 +197,19 @@ func deleteService(service *kapi.Service, inport, gwBridge string, nodeIP *net.I
 				klog.Errorf("Unable to generate cookie for svc: %s, %s, %d, error: %v",
 					service.Name, ingIP.String(), svcPort.Port, err)
 			}
-			_, stderr, err := util.RunOVSOfctl("del-flows", gwBridge,
+			_, stderr, err := util.RunOVSOfctl("del-flows", npw.gwBridge,
 				fmt.Sprintf("cookie=%s/-1", cookie))
 			if err != nil {
 				klog.Errorf("Failed to delete openflow flow on %s for Ingress: "+
-					"%s, stderr: %q, error: %v", gwBridge, ingIP, stderr, err)
+					"%s, stderr: %q, error: %v", npw.gwBridge, ingIP, stderr, err)
 			}
 		}
 	}
 
-	delSharedGatewayIptRules(service, nodeIP)
+	delSharedGatewayIptRules(service, npw.nodeIP)
 }
 
-func syncServices(services []interface{}, inport, gwBridge string, nodeIP *net.IPNet) {
+func (npw *nodePortWatcher) SyncServices(services []interface{}) {
 	ports := make(map[string]string)
 	ingressCookies := make(map[string]struct{})
 	for _, serviceInterface := range services {
@@ -219,30 +220,21 @@ func syncServices(services []interface{}, inport, gwBridge string, nodeIP *net.I
 			continue
 		}
 
-		if !util.ServiceTypeHasNodePort(service) && len(service.Spec.ExternalIPs) == 0 {
-			continue
-		}
-
 		for _, svcPort := range service.Spec.Ports {
-			if util.ServiceTypeHasNodePort(service) {
-				if err := util.ValidatePort(svcPort.Protocol, svcPort.NodePort); err != nil {
-					klog.Errorf("syncServices error for service port %s: %v", svcPort.Name, err)
-					continue
-				}
+			if svcPort.NodePort > 0 {
 				protocol := strings.ToLower(string(svcPort.Protocol))
 				nodePortKey := fmt.Sprintf("%s_%d", protocol, svcPort.NodePort)
 				ports[nodePortKey] = ""
 			}
 			for _, externalIP := range service.Spec.ExternalIPs {
-				if err := util.ValidatePort(svcPort.Protocol, svcPort.Port); err != nil {
-					klog.Errorf("syncServices error for service port %s: %v", svcPort.Name, err)
-					continue
-				}
 				protocol := strings.ToLower(string(svcPort.Protocol))
 				externalPortKey := fmt.Sprintf("%s_%d", protocol, svcPort.Port)
 				ports[externalPortKey] = externalIP
 			}
 			for _, ing := range service.Status.LoadBalancer.Ingress {
+				if ing.IP == "" {
+					continue
+				}
 				ingIP := net.ParseIP(ing.IP)
 				if ingIP == nil {
 					klog.Errorf("Failed to parse ingress IP: %s", ing.IP)
@@ -258,10 +250,10 @@ func syncServices(services []interface{}, inport, gwBridge string, nodeIP *net.I
 		}
 	}
 
-	syncSharedGatewayIptRules(services, nodeIP)
+	syncSharedGatewayIptRules(services, npw.nodeIP)
 
 	stdout, stderr, err := util.RunOVSOfctl("dump-flows",
-		gwBridge)
+		npw.gwBridge)
 	if err != nil {
 		klog.Errorf("dump-flows failed: %q (%v)", stderr, err)
 		return
@@ -311,12 +303,12 @@ func syncServices(services []interface{}, inport, gwBridge string, nodeIP *net.I
 					}
 				}
 				stdout, _, err := util.RunOVSOfctl(
-					"del-flows", gwBridge,
+					"del-flows", npw.gwBridge,
 					fmt.Sprintf("in_port=%s, %s, tp_dst=%s",
-						inport, protocol, port))
+						npw.ofportPhys, protocol, port))
 				if err != nil {
 					klog.Errorf("del-flows of %s failed: %q",
-						gwBridge, stdout)
+						npw.gwBridge, stdout)
 				}
 			} else {
 				nw_dst := "nw_dst"
@@ -326,105 +318,54 @@ func syncServices(services []interface{}, inport, gwBridge string, nodeIP *net.I
 					flowProtocol = protocol + "6"
 				}
 				stdout, _, err := util.RunOVSOfctl(
-					"del-flows", gwBridge,
+					"del-flows", npw.gwBridge,
 					fmt.Sprintf("in_port=%s, %s, %s=%s, tp_dst=%s",
-						inport, flowProtocol, nw_dst, externalIP, port))
+						npw.ofportPhys, flowProtocol, nw_dst, externalIP, port))
 				if err != nil {
 					klog.Errorf("del-flows of %s failed: %q",
-						gwBridge, stdout)
+						npw.gwBridge, stdout)
 				}
 			}
 		}
 	}
 }
 
-func nodePortWatcher(nodeName, gwBridge, gwIntf string, nodeIP []*net.IPNet, wf *factory.WatchFactory) error {
-	// the name of the patch port created by ovn-controller is of the form
-	// patch-<logical_port_name_of_localnet_port>-to-br-int
-	patchPort := "patch-" + gwBridge + "_" + nodeName + "-to-br-int"
-	// Get ofport of patchPort
-	ofportPatch, stderr, err := util.RunOVSVsctl("--if-exists", "get",
-		"interface", patchPort, "ofport")
-	if err != nil {
-		return fmt.Errorf("failed to get ofport of %s, stderr: %q, error: %v",
-			patchPort, stderr, err)
-	}
-
-	// Get ofport of physical interface
-	ofportPhys, stderr, err := util.RunOVSVsctl("--if-exists", "get",
-		"interface", gwIntf, "ofport")
-	if err != nil {
-		return fmt.Errorf("failed to get ofport of %s, stderr: %q, error: %v",
-			gwIntf, stderr, err)
-	}
-
-	// In the shared gateway mode, the NodePort service is handled by the OpenFlow flows configured
-	// on the OVS bridge in the host. These flows act only on the packets coming in from outside
-	// of the node. If someone on the node is trying to access the NodePort service, those packets
-	// will not be processed by the OpenFlow flows, so we need to add iptable rules that DNATs the
-	// NodePortIP:NodePort to ClusterServiceIP:Port.
-	if err := initSharedGatewayIPTables(); err != nil {
-		return err
-	}
-
-	wf.AddServiceHandler(cache.ResourceEventHandlerFuncs{
-		AddFunc: func(obj interface{}) {
-			service := obj.(*kapi.Service)
-			addService(service, ofportPhys, ofportPatch, gwBridge, nodeIP[0])
-		},
-		UpdateFunc: func(old, new interface{}) {
-			svcNew := new.(*kapi.Service)
-			svcOld := old.(*kapi.Service)
-			if reflect.DeepEqual(svcNew.Spec, svcOld.Spec) && reflect.DeepEqual(svcNew.Status, svcOld.Status) {
-				return
-			}
-			deleteService(svcOld, ofportPhys, gwBridge, nodeIP[0])
-			addService(svcNew, ofportPhys, ofportPatch, gwBridge, nodeIP[0])
-		},
-		DeleteFunc: func(obj interface{}) {
-			service := obj.(*kapi.Service)
-			deleteService(service, ofportPhys, gwBridge, nodeIP[0])
-		},
-	}, func(services []interface{}) {
-		syncServices(services, ofportPhys, gwBridge, nodeIP[0])
-	})
-
-	return nil
-}
-
 // since we share the host's k8s node IP, add OpenFlow flows
 // -- to steer the NodePort traffic arriving on the host to the OVN logical topology and
 // -- to also connection track the outbound north-south traffic through l3 gateway so that
 //    the return traffic can be steered back to OVN logical topology
 // -- to also handle unDNAT return traffic back out of the host
-func addDefaultConntrackRules(nodeName, gwBridge, gwIntf string, stopChan chan struct{}) error {
-	// the name of the patch port created by ovn-controller is of the form
-	// patch-<logical_port_name_of_localnet_port>-to-br-int
-	localnetLpName := gwBridge + "_" + nodeName
-	patchPort := "patch-" + localnetLpName + "-to-br-int"
-	// Get ofport of patchPort, but before that make sure ovn-controller created
-	// one for us (waits for about ovsCommandTimeout seconds)
-	ofportPatch, stderr, err := util.RunOVSVsctl("wait-until", "Interface", patchPort, "ofport>0",
-		"--", "get", "Interface", patchPort, "ofport")
+func newSharedGatewayOpenFlowManager(patchPort, macAddress, gwBridge, gwIntf string) (*openflowManager, error) {
+	// Get ofport of patchPort
+	ofportPatch, stderr, err := util.RunOVSVsctl("get", "Interface", patchPort, "ofport")
 	if err != nil {
-		return fmt.Errorf("failed while waiting on patch port %q to be created by ovn-controller and "+
+		return nil, fmt.Errorf("failed while waiting on patch port %q to be created by ovn-controller and "+
 			"while getting ofport. stderr: %q, error: %v", patchPort, stderr, err)
 	}
 
 	// Get ofport of physical interface
 	ofportPhys, stderr, err := util.RunOVSVsctl("get", "interface", gwIntf, "ofport")
 	if err != nil {
-		return fmt.Errorf("failed to get ofport of %s, stderr: %q, error: %v",
+		return nil, fmt.Errorf("failed to get ofport of %s, stderr: %q, error: %v",
 			gwIntf, stderr, err)
 	}
 
-	// replace the left over OpenFlow flows with the FLOOD action flow
-	_, stderr, err = util.AddFloodActionOFFlow(gwBridge)
+	// replace the left over OpenFlow flows with the NORMAL action flow
+	_, stderr, err = util.AddOFFlowWithSpecificAction(gwBridge, util.NormalAction)
 	if err != nil {
-		return fmt.Errorf("failed to replace-flows on bridge %q stderr:%s (%v)", gwBridge, stderr, err)
+		return nil, fmt.Errorf("failed to replace-flows on bridge %q stderr:%s (%v)", gwBridge, stderr, err)
 	}
 
 	nFlows := 0
+	// table 0, we check to see if this dest mac is the shared mac, if so flood to both ports
+	_, stderr, err = util.RunOVSOfctl("add-flow", gwBridge,
+		fmt.Sprintf("cookie=%s, priority=10, table=0, in_port=%s, dl_dst=%s, actions=output:%s,output:LOCAL",
+			defaultOpenFlowCookie, ofportPhys, macAddress, ofportPatch))
+	if err != nil {
+		return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, error: %v", gwBridge, stderr, err)
+	}
+	nFlows++
+
 	if config.IPv4Mode {
 		// table 0, packets coming from pods headed externally. Commit connections
 		// so that reverse direction goes back to the pods.
@@ -433,7 +374,7 @@ func addDefaultConntrackRules(nodeName, gwBridge, gwIntf string, stopChan chan s
 				"actions=ct(commit, zone=%d), output:%s",
 				defaultOpenFlowCookie, ofportPatch, config.Default.ConntrackZone, ofportPhys))
 		if err != nil {
-			return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+			return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
 				"error: %v", gwBridge, stderr, err)
 		}
 		nFlows++
@@ -444,7 +385,7 @@ func addDefaultConntrackRules(nodeName, gwBridge, gwIntf string, stopChan chan s
 			fmt.Sprintf("cookie=%s, priority=50, in_port=%s, ip, "+
 				"actions=ct(zone=%d, table=1)", defaultOpenFlowCookie, ofportPhys, config.Default.ConntrackZone))
 		if err != nil {
-			return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+			return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
 				"error: %v", gwBridge, stderr, err)
 		}
 		nFlows++
@@ -457,7 +398,7 @@ func addDefaultConntrackRules(nodeName, gwBridge, gwIntf string, stopChan chan s
 				"actions=ct(commit, zone=%d), output:%s",
 				defaultOpenFlowCookie, ofportPatch, config.Default.ConntrackZone, ofportPhys))
 		if err != nil {
-			return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+			return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
 				"error: %v", gwBridge, stderr, err)
 		}
 		nFlows++
@@ -468,7 +409,7 @@ func addDefaultConntrackRules(nodeName, gwBridge, gwIntf string, stopChan chan s
 			fmt.Sprintf("cookie=%s, priority=50, in_port=%s, ipv6, "+
 				"actions=ct(zone=%d, table=1)", defaultOpenFlowCookie, ofportPhys, config.Default.ConntrackZone))
 		if err != nil {
-			return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+			return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
 				"error: %v", gwBridge, stderr, err)
 		}
 		nFlows++
@@ -479,7 +420,7 @@ func addDefaultConntrackRules(nodeName, gwBridge, gwIntf string, stopChan chan s
 		fmt.Sprintf("cookie=%s, priority=100, table=1, ct_state=+trk+est, "+
 			"actions=output:%s", defaultOpenFlowCookie, ofportPatch))
 	if err != nil {
-		return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+		return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
 			"error: %v", gwBridge, stderr, err)
 	}
 	nFlows++
@@ -488,129 +429,147 @@ func addDefaultConntrackRules(nodeName, gwBridge, gwIntf string, stopChan chan s
 		fmt.Sprintf("cookie=%s, priority=100, table=1, ct_state=+trk+rel, "+
 			"actions=output:%s", defaultOpenFlowCookie, ofportPatch))
 	if err != nil {
-		return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+		return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
 			"error: %v", gwBridge, stderr, err)
 	}
 	nFlows++
 
+	if config.Gateway.DisableSNATMultipleGWs {
+		// table 1, traffic to pod subnet go directly to OVN
+		for _, clusterEntry := range config.Default.ClusterSubnets {
+			cidr := clusterEntry.CIDR
+			var ipPrefix string
+			if utilnet.IsIPv6CIDR(cidr) {
+				ipPrefix = "ipv6"
+			} else {
+				ipPrefix = "ip"
+			}
+			_, stderr, err = util.RunOVSOfctl("add-flow", gwBridge,
+				fmt.Sprintf("cookie=%s, priority=15, table=1, %s, %s_dst=%s, actions=output:%s",
+					defaultOpenFlowCookie, ipPrefix, ipPrefix, cidr, ofportPatch))
+			if err != nil {
+				return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+					"error: %v", gwBridge, stderr, err)
+			}
+			nFlows++
+		}
+	}
+
+	// table 1, we check to see if this dest mac is the shared mac, if so send to host
+	_, stderr, err = util.RunOVSOfctl("add-flow", gwBridge,
+		fmt.Sprintf("cookie=%s, priority=10, table=1, dl_dst=%s, actions=output:LOCAL",
+			defaultOpenFlowCookie, macAddress))
+	if err != nil {
+		return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, error: %v", gwBridge, stderr, err)
+	}
+	nFlows++
+
+	if config.IPv6Mode {
+		// REMOVEME(trozet) when https://bugzilla.kernel.org/show_bug.cgi?id=11797 is resolved
+		// must flood icmpv6 Route Advertisement and Neighbor Advertisement traffic as it fails to create a CT entry
+		for _, icmpType := range []int{types.RouteAdvertisementICMPType, types.NeighborAdvertisementICMPType} {
+			_, stderr, err = util.RunOVSOfctl("add-flow", gwBridge,
+				fmt.Sprintf("cookie=%s, priority=14, table=1,icmp6,icmpv6_type=%d actions=FLOOD",
+					defaultOpenFlowCookie, icmpType))
+			if err != nil {
+				return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+					"error: %v", gwBridge, stderr, err)
+			}
+			nFlows++
+		}
+	}
+
 	// table 1, all other connections do normal processing
 	_, stderr, err = util.RunOVSOfctl("add-flow", gwBridge,
-		fmt.Sprintf("cookie=%s, priority=0, table=1, actions=output:FLOOD", defaultOpenFlowCookie))
+		fmt.Sprintf("cookie=%s, priority=0, table=1, actions=output:NORMAL", defaultOpenFlowCookie))
 	if err != nil {
-		return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
-			"error: %v", gwBridge, stderr, err)
-	}
-	nFlows++
-
-	// table 2, return traffic to go to out of the host from nodePort or load balancer access
-	_, stderr, err = util.RunOVSOfctl("add-flow", gwBridge,
-		fmt.Sprintf("cookie=%s, priority=0, table=2, actions=output:%s", defaultOpenFlowCookie, ofportPhys))
-	if err != nil {
-		return fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
+		return nil, fmt.Errorf("failed to add openflow flow to %s, stderr: %q, "+
 			"error: %v", gwBridge, stderr, err)
 	}
 	nFlows++
 
 	// add health check function to check default OpenFlow flows are on the shared gateway bridge
-	go checkDefaultConntrackRules(gwBridge, gwIntf, patchPort, ofportPhys, ofportPatch, nFlows, stopChan)
-	return nil
+	return &openflowManager{
+		gwBridge, gwIntf, patchPort, ofportPhys, ofportPatch, nFlows,
+	}, nil
 }
 
-func (n *OvnNode) initSharedGateway(subnets []*net.IPNet, gwNextHops []net.IP, gwIntf string, nodeAnnotator kube.Annotator) (postWaitFunc, error) {
-	var bridgeName string
-	var uplinkName string
-	var brCreated bool
-	var err error
+func newSharedGateway(nodeName string, subnets []*net.IPNet, gwNextHops []net.IP, gwIntf string, nodeAnnotator kube.Annotator) (*gateway, error) {
+	klog.Info("Creating new shared gateway")
+	gw := &gateway{}
 
-	// OCP HACK
-	// Do not configure OVS bridge for local gateway mode with a gateway iface of none
-	// For 4.5->4.6 migration, see 	https://github.com/openshift/ovn-kubernetes/pull/281
-	if gwIntf == "none" {
-		return n.initSharedGatewayNoBridge(subnets, gwNextHops, nodeAnnotator)
-		// END OCP HACK
-	} else if bridgeName, _, err = util.RunOVSVsctl("--", "port-to-br", gwIntf); err == nil {
-		// This is an OVS bridge's internal port
-		uplinkName, err = util.GetNicName(bridgeName)
-		if err != nil {
-			return nil, err
-		}
-	} else if _, _, err := util.RunOVSVsctl("--", "br-exists", gwIntf); err != nil {
-		// This is not a OVS bridge. We need to create a OVS bridge
-		// and add cluster.GatewayIntf as a port of that bridge.
-		bridgeName, err = util.NicToBridge(gwIntf)
-		if err != nil {
-			return nil, fmt.Errorf("failed to convert %s to OVS bridge: %v",
-				gwIntf, err)
-		}
-		uplinkName = gwIntf
-		gwIntf = bridgeName
-		brCreated = true
-	} else {
-		// gateway interface is an OVS bridge
-		uplinkName, err = getIntfName(gwIntf)
-		if err != nil {
-			return nil, err
-		}
-		bridgeName = gwIntf
-	}
-
-	// Now, we get IP addresses from OVS bridge. If IP does not exist,
-	// error out.
-	ips, err := getNetworkInterfaceIPAddresses(gwIntf)
-	if err != nil {
-		return nil, fmt.Errorf("failed to get interface details for %s (%v)",
-			gwIntf, err)
-	}
-	ifaceID, macAddress, err := bridgedGatewayNodeSetup(n.name, bridgeName, gwIntf,
-		util.PhysicalNetworkName, brCreated)
-	if err != nil {
-		return nil, fmt.Errorf("failed to set up shared interface gateway: %v", err)
-	}
-
-	err = setupLocalNodeAccessBridge(n.name, subnets)
-	if err != nil {
-		return nil, err
-	}
-	chassisID, err := util.GetNodeChassisID()
+	bridgeName, uplinkName, macAddress, ips, err := gatewayInitInternal(
+		nodeName, gwIntf, subnets, gwNextHops, nodeAnnotator)
 	if err != nil {
 		return nil, err
 	}
 
-	err = util.SetL3GatewayConfig(nodeAnnotator, &util.L3GatewayConfig{
-		Mode:           config.GatewayModeShared,
-		ChassisID:      chassisID,
-		InterfaceID:    ifaceID,
-		MACAddress:     macAddress,
-		IPAddresses:    ips,
-		NextHops:       gwNextHops,
-		NodePortEnable: config.Gateway.NodeportEnable,
-		VLANID:         &config.Gateway.VLANID,
-	})
-	if err != nil {
-		return nil, err
+	// the name of the patch port created by ovn-controller is of the form
+	// patch-<logical_port_name_of_localnet_port>-to-br-int
+	patchPort := "patch-" + bridgeName + "_" + nodeName + "-to-br-int"
+
+	gw.readyFunc = func() (bool, error) {
+		return gatewayReady(patchPort)
 	}
 
-	return func() error {
+	gw.initFunc = func() error {
+		// Program cluster.GatewayIntf to let non-pod traffic to go to host
+		// stack
+		klog.Info("Creating Shared Gateway Openflow Manager")
+		var err error
+
+		gw.openflowManager, err = newSharedGatewayOpenFlowManager(patchPort, macAddress.String(), bridgeName, uplinkName)
+		if err != nil {
+			return err
+		}
+
 		if config.Gateway.NodeportEnable {
-			if config.Gateway.Mode == config.GatewayModeLocal {
-				if err := addDefaultConntrackRulesLocal(n.name, bridgeName, uplinkName, n.stopChan); err != nil {
-					return err
-				}
-			} else {
-				// Program cluster.GatewayIntf to let non-pod traffic to go to host
-				// stack
-				if err := addDefaultConntrackRules(n.name, bridgeName, uplinkName, n.stopChan); err != nil {
-					return err
-				}
-				// Program cluster.GatewayIntf to let nodePort traffic to go to pods.
-				if err := nodePortWatcher(n.name, bridgeName, uplinkName, ips,
-					n.watchFactory); err != nil {
-					return err
-				}
+			klog.Info("Creating Shared Gateway Node Port Watcher")
+			gw.nodePortWatcher, err = newNodePortWatcher(patchPort, bridgeName, uplinkName, ips[0])
+			if err != nil {
+				return err
 			}
 		}
 		return nil
-	}, nil
+	}
+
+	klog.Info("Shared Gateway Creation Complete")
+	return gw, nil
+}
+
+func newNodePortWatcher(patchPort, gwBridge, gwIntf string, nodeIP *net.IPNet) (*nodePortWatcher, error) {
+	// Get ofport of patchPort
+	ofportPatch, stderr, err := util.RunOVSVsctl("--if-exists", "get",
+		"interface", patchPort, "ofport")
+	if err != nil {
+		return nil, fmt.Errorf("failed to get ofport of %s, stderr: %q, error: %v",
+			patchPort, stderr, err)
+	}
+
+	// Get ofport of physical interface
+	ofportPhys, stderr, err := util.RunOVSVsctl("--if-exists", "get",
+		"interface", gwIntf, "ofport")
+	if err != nil {
+		return nil, fmt.Errorf("failed to get ofport of %s, stderr: %q, error: %v",
+			gwIntf, stderr, err)
+	}
+
+	// In the shared gateway mode, the NodePort service is handled by the OpenFlow flows configured
+	// on the OVS bridge in the host. These flows act only on the packets coming in from outside
+	// of the node. If someone on the node is trying to access the NodePort service, those packets
+	// will not be processed by the OpenFlow flows, so we need to add iptable rules that DNATs the
+	// NodePortIP:NodePort to ClusterServiceIP:Port.
+	if err := initSharedGatewayIPTables(); err != nil {
+		return nil, err
+	}
+
+	npw := &nodePortWatcher{
+		ofportPhys:  ofportPhys,
+		ofportPatch: ofportPatch,
+		gwBridge:    gwBridge,
+		nodeIP:      nodeIP,
+	}
+	return npw, nil
 }
 
 func cleanupSharedGateway() error {
@@ -639,7 +598,7 @@ func cleanupSharedGateway() error {
 	bridgeMappings := strings.Split(stdout, ",")
 	for _, bridgeMapping := range bridgeMappings {
 		m := strings.Split(bridgeMapping, ":")
-		if network := m[0]; network == util.PhysicalNetworkName {
+		if network := m[0]; network == types.PhysicalNetworkName {
 			bridgeName = m[1]
 			break
 		}
@@ -648,7 +607,7 @@ func cleanupSharedGateway() error {
 		return nil
 	}
 
-	_, stderr, err = util.AddFloodActionOFFlow(bridgeName)
+	_, stderr, err = util.AddOFFlowWithSpecificAction(bridgeName, util.NormalAction)
 	if err != nil {
 		return fmt.Errorf("failed to replace-flows on bridge %q stderr:%s (%v)", bridgeName, stderr, err)
 	}
