@@ -7,6 +7,7 @@ import (
 	"time"
 
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/types"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
 	kapi "k8s.io/api/core/v1"
@@ -16,10 +17,16 @@ import (
 )
 
 // gatewayInit creates a gateway router for the local chassis.
-func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*net.IPNet, joinSubnets []*net.IPNet,
-	l3GatewayConfig *util.L3GatewayConfig, sctpSupport bool) error {
+func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*net.IPNet,
+	l3GatewayConfig *util.L3GatewayConfig, sctpSupport bool, gwLRPIfAddrs, drLRPIfAddrs []*net.IPNet) error {
+
+	gwLRPIPs := make([]net.IP, 0)
+	for _, gwLRPIfAddr := range gwLRPIfAddrs {
+		gwLRPIPs = append(gwLRPIPs, gwLRPIfAddr.IP)
+	}
+
 	// Create a gateway router.
-	gatewayRouter := gwRouterPrefix + nodeName
+	gatewayRouter := types.GWRouterPrefix + nodeName
 	physicalIPs := make([]string, len(l3GatewayConfig.IPAddresses))
 	for i, ip := range l3GatewayConfig.IPAddresses {
 		physicalIPs[i] = ip.IP.String()
@@ -34,80 +41,32 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 			"stderr: %q, error: %v", gatewayRouter, stdout, stderr, err)
 	}
 
-	var gwLRPMAC, drLRPMAC net.HardwareAddr
-	var gwLRPIPs, drLRPIPs []net.IP
-	var gwLRPAddrs, drLRPAddrs []string
+	gwSwitchPort := types.JoinSwitchToGWRouterPrefix + gatewayRouter
+	gwRouterPort := types.GWRouterToJoinSwitchPrefix + gatewayRouter
 
-	for _, joinSubnet := range joinSubnets {
-		prefixLen, _ := joinSubnet.Mask.Size()
-		gwLRPIP := util.NextIP(joinSubnet.IP)
-		gwLRPIPs = append(gwLRPIPs, gwLRPIP)
-		gwLRPAddrs = append(gwLRPAddrs, fmt.Sprintf("%s/%d", gwLRPIP.String(), prefixLen))
-		drLRPIP := util.NextIP(gwLRPIP)
-		drLRPIPs = append(drLRPIPs, drLRPIP)
-		drLRPAddrs = append(drLRPAddrs, fmt.Sprintf("%s/%d", drLRPIP.String(), prefixLen))
-
-		if gwLRPMAC == nil || !utilnet.IsIPv6(gwLRPIP) {
-			gwLRPMAC = util.IPAddrToHWAddr(gwLRPIP)
-			drLRPMAC = util.IPAddrToHWAddr(drLRPIP)
-		}
-	}
-
-	joinSwitch := joinSwitchPrefix + nodeName
-	// create the per-node join switch
-	stdout, stderr, err = util.RunOVNNbctl("--", "--may-exist", "ls-add", joinSwitch)
-	if err != nil {
-		return fmt.Errorf("failed to create logical switch %q, stdout: %q, stderr: %q, error: %v",
-			joinSwitch, stdout, stderr, err)
-	}
-
-	gwSwitchPort := joinSwitchToGwRouterPrefix + gatewayRouter
-	gwRouterPort := gwRouterToJoinSwitchPrefix + gatewayRouter
 	stdout, stderr, err = util.RunOVNNbctl(
-		"--", "--may-exist", "lsp-add", joinSwitch, gwSwitchPort,
+		"--", "--may-exist", "lsp-add", types.OVNJoinSwitch, gwSwitchPort,
 		"--", "set", "logical_switch_port", gwSwitchPort, "type=router", "options:router-port="+gwRouterPort,
 		"addresses=router")
 	if err != nil {
 		return fmt.Errorf("failed to add port %q to logical switch %q, "+
-			"stdout: %q, stderr: %q, error: %v", gwSwitchPort, joinSwitch, stdout, stderr, err)
+			"stdout: %q, stderr: %q, error: %v", gwSwitchPort, types.OVNJoinSwitch, stdout, stderr, err)
 	}
 
+	gwLRPMAC := util.IPAddrToHWAddr(gwLRPIPs[0])
 	args := []string{
 		"--", "--if-exists", "lrp-del", gwRouterPort,
 		"--", "lrp-add", gatewayRouter, gwRouterPort, gwLRPMAC.String(),
 	}
-	args = append(args, gwLRPAddrs...)
+	for _, gwLRPIfAddr := range gwLRPIfAddrs {
+		args = append(args, gwLRPIfAddr.String())
+	}
 	_, stderr, err = util.RunOVNNbctl(args...)
 	if err != nil {
 		return fmt.Errorf("failed to add logical router port %q for gateway router %s, "+
 			"stderr: %q, error: %v", gwRouterPort, gatewayRouter, stderr, err)
 	}
 
-	// jtod/dtoj - patch ports that connect the per-node join switch to distributed router
-	drSwitchPort := joinSwitchToDistRouterPrefix + nodeName
-	drRouterPort := distRouterToJoinSwitchPrefix + nodeName
-
-	// Connect the per-node join switch to the distributed router.
-	stdout, stderr, err = util.RunOVNNbctl(
-		"--", "--may-exist", "lsp-add", joinSwitch, drSwitchPort,
-		"--", "set", "logical_switch_port", drSwitchPort, "type=router", "options:router-port="+drRouterPort,
-		"addresses=router")
-	if err != nil {
-		return fmt.Errorf("failed to add port %q to logical switch %q, "+
-			"stdout: %q, stderr: %q, error: %v", drSwitchPort, joinSwitch, stdout, stderr, err)
-	}
-
-	args = []string{
-		"--", "--if-exists", "lrp-del", drRouterPort,
-		"--", "lrp-add", ovnClusterRouter, drRouterPort, drLRPMAC.String(),
-	}
-	args = append(args, drLRPAddrs...)
-	_, stderr, err = util.RunOVNNbctl(args...)
-	if err != nil {
-		return fmt.Errorf("failed to add logical router port %q to %s, "+
-			"stderr: %q, error: %v", drRouterPort, ovnClusterRouter, stderr, err)
-	}
-
 	// Local gateway mode does not need SNAT or routes on GR because GR is only used for multiple external gws
 	// without SNAT. For normal N/S traffic, ingress/egress is mp0 on node switches
 	if config.Gateway.Mode != config.GatewayModeLocal {
@@ -121,10 +80,37 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 			return fmt.Errorf("failed to set logical router %s's lb_force_snat_ip option, "+
 				"stdout: %q, stderr: %q, error: %v", gatewayRouter, stdout, stderr, err)
 		}
+
+		// Set shared gw SNAT to use host CT Zone. This will avoid potential SNAT collisions between
+		// host networked pods and OVN networked pods northbound traffic
+		stdout, stderr, err = util.RunOVNNbctl("set", "logical_router", gatewayRouter, "options:snat-ct-zone=0")
+		if err != nil {
+			return fmt.Errorf("failed to set logical router %s's SNAT CT Zone to 0, "+
+				"stdout: %q, stderr: %q, error: %v", gatewayRouter, stdout, stderr, err)
+		}
+	}
+
+	// To decrease the numbers of MAC_Binding entries in a large scale cluster, change the default behavior of always
+	// learning the MAC/IP binding and adding a new MAC_Binding entry. Only do it when necessary.
+	// See details in ovn-northd(8).
+	stdout, stderr, err = util.RunOVNNbctl("set", "logical_router", gatewayRouter,
+		"options:always_learn_from_arp_request=false")
+	if err != nil {
+		klog.Warningf("Failed to set logical router %s's always_learn_from_arp_request "+
+			"stdout: %q, stderr: %q, error: %v", gatewayRouter, stdout, stderr, err)
+	}
+
+	// To avoid flow exploding problem in large scale ovn-kubernetes cluster, not to prepopulate static mappings
+	// for all neighbor routers in the ARP/ND Resolution stage. See details in ovn-northd(8).
+	stdout, stderr, err = util.RunOVNNbctl("set", "logical_router",
+		gatewayRouter, "options:dynamic_neigh_routers=true")
+	if err != nil {
+		klog.Warningf("Failed to set logical router %s's dynamic_neigh_routers "+
+			"stdout: %q, stderr: %q, error: %v", gatewayRouter, stdout, stderr, err)
 	}
 
 	for _, entry := range clusterIPSubnet {
-		drLRPIP, err := util.MatchIPFamily(utilnet.IsIPv6CIDR(entry), drLRPIPs)
+		drLRPIfAddr, err := util.MatchIPNetFamily(utilnet.IsIPv6CIDR(entry), drLRPIfAddrs)
 		if err != nil {
 			return fmt.Errorf("failed to add a static route in GR %s with distributed "+
 				"router as the nexthop: %v",
@@ -133,7 +119,7 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 
 		// Add a static route in GR with distributed router as the nexthop.
 		stdout, stderr, err = util.RunOVNNbctl("--may-exist", "lr-route-add",
-			gatewayRouter, entry.String(), drLRPIP.String())
+			gatewayRouter, entry.String(), drLRPIfAddr.IP.String())
 		if err != nil {
 			return fmt.Errorf("failed to add a static route in GR %s with distributed "+
 				"router as the nexthop, stdout: %q, stderr: %q, error: %v",
@@ -204,7 +190,7 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 	}
 
 	// Create the external switch for the physical interface to connect to.
-	externalSwitch := externalSwitchPrefix + nodeName
+	externalSwitch := types.ExternalSwitchPrefix + nodeName
 	stdout, stderr, err = util.RunOVNNbctl("--may-exist", "ls-add",
 		externalSwitch)
 	if err != nil {
@@ -212,10 +198,6 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 			"stderr: %q, error: %v", externalSwitch, stdout, stderr, err)
 	}
 
-	// OCP HACK - clean up old local gateway port that might still be in DB
-	cleanOldLocalGWPort(nodeName)
-	// END OCP HACK
-
 	// Add external interface as a logical port to external_switch.
 	// This is a learning switch port with "unknown" address. The external
 	// world is accessed via this port.
@@ -223,7 +205,7 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 		"--", "--may-exist", "lsp-add", externalSwitch, l3GatewayConfig.InterfaceID,
 		"--", "lsp-set-addresses", l3GatewayConfig.InterfaceID, "unknown",
 		"--", "lsp-set-type", l3GatewayConfig.InterfaceID, "localnet",
-		"--", "lsp-set-options", l3GatewayConfig.InterfaceID, "network_name=" + util.PhysicalNetworkName}
+		"--", "lsp-set-options", l3GatewayConfig.InterfaceID, "network_name=" + types.PhysicalNetworkName}
 
 	if l3GatewayConfig.VLANID != nil {
 		lspArgs := []string{
@@ -245,15 +227,15 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 	// direct addition of logical_router_port with --may-exists will not work since the MAC
 	// has changed. So, we need to delete that port, if it exists, and it back.
 	cmdArgs = []string{
-		"--", "--if-exists", "lrp-del", gwRouterToExtSwitchPrefix + gatewayRouter,
-		"--", "lrp-add", gatewayRouter, gwRouterToExtSwitchPrefix + gatewayRouter,
+		"--", "--if-exists", "lrp-del", types.GWRouterToExtSwitchPrefix + gatewayRouter,
+		"--", "lrp-add", gatewayRouter, types.GWRouterToExtSwitchPrefix + gatewayRouter,
 		l3GatewayConfig.MACAddress.String(),
 	}
 	for _, ip := range l3GatewayConfig.IPAddresses {
 		cmdArgs = append(cmdArgs, ip.String())
 	}
 	cmdArgs = append(cmdArgs,
-		"--", "set", "logical_router_port", gwRouterToExtSwitchPrefix+gatewayRouter,
+		"--", "set", "logical_router_port", types.GWRouterToExtSwitchPrefix+gatewayRouter,
 		"external-ids:gateway-physical-ip=yes")
 
 	stdout, stderr, err = util.RunOVNNbctl(cmdArgs...)
@@ -264,9 +246,9 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 
 	// Connect the external_switch to the router.
 	stdout, stderr, err = util.RunOVNNbctl("--", "--may-exist", "lsp-add",
-		externalSwitch, extSwitchToGwRouterPrefix+gatewayRouter, "--", "set",
-		"logical_switch_port", extSwitchToGwRouterPrefix+gatewayRouter, "type=router",
-		"options:router-port="+gwRouterToExtSwitchPrefix+gatewayRouter,
+		externalSwitch, types.EXTSwitchToGWRouterPrefix+gatewayRouter, "--", "set",
+		"logical_switch_port", types.EXTSwitchToGWRouterPrefix+gatewayRouter, "type=router",
+		"options:router-port="+types.GWRouterToExtSwitchPrefix+gatewayRouter,
 		"addresses="+"\""+l3GatewayConfig.MACAddress.String()+"\"")
 	if err != nil {
 		return fmt.Errorf("failed to add logical port to router %s, stdout: %q, "+
@@ -283,7 +265,7 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 		}
 		stdout, stderr, err = util.RunOVNNbctl("--may-exist", "lr-route-add",
 			gatewayRouter, allIPs, nextHop.String(),
-			fmt.Sprintf("rtoe-%s", gatewayRouter))
+			fmt.Sprintf("%s%s", types.GWRouterToExtSwitchPrefix, gatewayRouter))
 		if err != nil {
 			return fmt.Errorf("failed to add a static route in GR %s with physical "+
 				"gateway as the default next hop, stdout: %q, "+
@@ -291,6 +273,21 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 		}
 	}
 
+	// We need to add a route to the Gateway router's IP, on the
+	// cluster router, to ensure that the return traffic goes back
+	// to the same gateway router
+	//
+	// This can be removed once https://bugzilla.redhat.com/show_bug.cgi?id=1891516 is fixed.
+	for _, gwLRPIP := range gwLRPIPs {
+		stdout, stderr, err = util.RunOVNNbctl("--may-exist", "lr-route-add",
+			types.OVNClusterRouter, gwLRPIP.String(), gwLRPIP.String())
+		if err != nil {
+			return fmt.Errorf("failed to add the route to Gateway router's IP of %q "+
+				"on the distributed router, stdout: %q, stderr: %q, error: %v",
+				gwLRPIP.String(), stdout, stderr, err)
+		}
+	}
+
 	// Add source IP address based routes in distributed router
 	// for this gateway router.
 	for _, hostSubnet := range hostSubnets {
@@ -298,23 +295,23 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 		if err != nil {
 			return fmt.Errorf("failed to add source IP address based "+
 				"routes in distributed router %s: %v",
-				ovnClusterRouter, err)
+				types.OVNClusterRouter, err)
 		}
 
 		if config.Gateway.Mode != config.GatewayModeLocal {
 			stdout, stderr, err = util.RunOVNNbctl("--may-exist",
-				"--policy=src-ip", "lr-route-add", ovnClusterRouter,
-				hostSubnet.String(), gwLRPIP.String())
+				"--policy=src-ip", "lr-route-add", types.OVNClusterRouter,
+				hostSubnet.String(), gwLRPIP[0].String())
 			if err != nil {
 				return fmt.Errorf("failed to add source IP address based "+
 					"routes in distributed router %s, stdout: %q, "+
-					"stderr: %q, error: %v", ovnClusterRouter, stdout, stderr, err)
+					"stderr: %q, error: %v", types.OVNClusterRouter, stdout, stderr, err)
 			}
 		}
 	}
 
-	// if config.Gateway.DisabledSNATMultipleGWs is not set (by default is not),
-	// the NAT rules for pods not having annotations to route thru either external
+	// if config.Gateway.DisabledSNATMultipleGWs is not set (by default it is not),
+	// the NAT rules for pods not having annotations to route through either external
 	// gws or pod CNFs will be added within pods.go addLogicalPort
 	if !config.Gateway.DisableSNATMultipleGWs && config.Gateway.Mode != config.GatewayModeLocal {
 		// Default SNAT rules.
@@ -328,10 +325,14 @@ func gatewayInit(nodeName string, clusterIPSubnet []*net.IPNet, hostSubnets []*n
 				return fmt.Errorf("failed to create default SNAT rules for gateway router %s: %v",
 					gatewayRouter, err)
 			}
-
-			if err := util.UpdateRouterSNAT(gatewayRouter, externalIP, entry); err != nil {
-				return fmt.Errorf("failed to update NAT entry for pod subnet: %s, GR: %s, error: %v",
-					entry.String(), gatewayRouter, err)
+			// delete the existing lr-nat rule first otherwise gateway init fails
+			// if the external ip has changed, but the logical ip has stayed the same
+			stdout, stderr, err := util.RunOVNNbctl("--if-exists", "lr-nat-del",
+				gatewayRouter, "snat", entry.String(), "--", "lr-nat-add",
+				gatewayRouter, "snat", externalIP[0].String(), entry.String())
+			if err != nil {
+				return fmt.Errorf("failed to create default SNAT rules for gateway router %s, "+
+					"stdout: %q, stderr: %q, error: %v", gatewayRouter, stdout, stderr, err)
 			}
 		}
 	}
@@ -347,27 +348,27 @@ func addDistributedGWPort() error {
 	var dgpMac string
 	var nbctlArgs []string
 	// add a distributed gateway port to the distributed router
-	dgpName := routerToSwitchPrefix + nodeLocalSwitch
+	dgpName := types.RouterToSwitchPrefix + types.NodeLocalSwitch
 	if config.IPv4Mode {
-		dgpMac = util.IPAddrToHWAddr(net.ParseIP(util.V4NodeLocalDistributedGwPortIP)).String()
+		dgpMac = util.IPAddrToHWAddr(net.ParseIP(types.V4NodeLocalDistributedGWPortIP)).String()
 	} else {
-		dgpMac = util.IPAddrToHWAddr(net.ParseIP(util.V6NodeLocalDistributedGwPortIP)).String()
+		dgpMac = util.IPAddrToHWAddr(net.ParseIP(types.V6NodeLocalDistributedGWPortIP)).String()
 	}
 	nbctlArgs = append(nbctlArgs,
-		"--may-exist", "lrp-add", ovnClusterRouter, dgpName, dgpMac,
+		"--may-exist", "lrp-add", types.OVNClusterRouter, dgpName, dgpMac,
 	)
 	if config.IPv4Mode && config.IPv6Mode {
 		nbctlArgs = append(nbctlArgs,
-			fmt.Sprintf("%s/%d", util.V4NodeLocalDistributedGwPortIP, util.V4NodeLocalNatSubnetPrefix),
-			fmt.Sprintf("%s/%d", util.V6NodeLocalDistributedGwPortIP, util.V6NodeLocalNatSubnetPrefix),
+			fmt.Sprintf("%s/%d", types.V4NodeLocalDistributedGWPortIP, types.V4NodeLocalNATSubnetPrefix),
+			fmt.Sprintf("%s/%d", types.V6NodeLocalDistributedGWPortIP, types.V6NodeLocalNATSubnetPrefix),
 		)
 	} else if config.IPv4Mode {
 		nbctlArgs = append(nbctlArgs,
-			fmt.Sprintf("%s/%d", util.V4NodeLocalDistributedGwPortIP, util.V4NodeLocalNatSubnetPrefix),
+			fmt.Sprintf("%s/%d", types.V4NodeLocalDistributedGWPortIP, types.V4NodeLocalNATSubnetPrefix),
 		)
 	} else if config.IPv6Mode {
 		nbctlArgs = append(nbctlArgs,
-			fmt.Sprintf("%s/%d", util.V6NodeLocalDistributedGwPortIP, util.V6NodeLocalNatSubnetPrefix),
+			fmt.Sprintf("%s/%d", types.V6NodeLocalDistributedGWPortIP, types.V6NodeLocalNATSubnetPrefix),
 		)
 	}
 	// set gateway chassis (the current master node) for distributed gateway port)
@@ -383,28 +384,28 @@ func addDistributedGWPort() error {
 
 	// connect the distributed gateway port to logical switch configured with localnet port
 	nbctlArgs = []string{
-		"--may-exist", "ls-add", nodeLocalSwitch,
+		"--may-exist", "ls-add", types.NodeLocalSwitch,
 	}
 	// add localnet port to the logical switch
-	lclNetPortname := "lnet-" + nodeLocalSwitch
+	lclNetPortname := "lnet-" + types.NodeLocalSwitch
 	nbctlArgs = append(nbctlArgs,
-		"--", "--may-exist", "lsp-add", nodeLocalSwitch, lclNetPortname,
+		"--", "--may-exist", "lsp-add", types.NodeLocalSwitch, lclNetPortname,
 		"--", "set", "logical_switch_port", lclNetPortname, "addresses=unknown", "type=localnet",
-		"options:network_name="+util.LocalNetworkName)
+		"options:network_name="+types.LocalNetworkName)
 	// connect the switch to the distributed router
-	lspName := switchToRouterPrefix + nodeLocalSwitch
+	lspName := types.SwitchToRouterPrefix + types.NodeLocalSwitch
 	nbctlArgs = append(nbctlArgs,
-		"--", "--may-exist", "lsp-add", nodeLocalSwitch, lspName,
+		"--", "--may-exist", "lsp-add", types.NodeLocalSwitch, lspName,
 		"--", "set", "logical_switch_port", lspName, "type=router", "addresses=router",
 		"options:nat-addresses=router", "options:router-port="+dgpName)
 	stdout, stderr, err = util.RunOVNNbctl(nbctlArgs...)
 	if err != nil {
 		return fmt.Errorf("failed creating logical switch %s and its ports (%s, %s) "+
-			"stdout: %q, stderr: %q, error: %v", nodeLocalSwitch, lclNetPortname, lspName, stdout, stderr, err)
+			"stdout: %q, stderr: %q, error: %v", types.NodeLocalSwitch, lclNetPortname, lspName, stdout, stderr, err)
 	}
 	// finally add an entry to the OVN SB MAC_Binding table, if not present, to capture the
-	// MAC-IP binding of util.V4NodeLocalNatSubnetNextHop address or
-	// util.V6NodeLocalNatSubnetNextHop address to its MAC. Normally, this will
+	// MAC-IP binding of types.V4NodeLocalNATSubnetNextHop address or
+	// types.V6NodeLocalNATSubnetNextHop address to its MAC. Normally, this will
 	// be learnt and added by the chassis to which the distributed gateway port (DGP) is
 	// bound. However, in our case we don't send any traffic out with the DGP port's IP
 	// as source IP, so that binding will never be learnt and we need to seed it.
@@ -412,14 +413,14 @@ func addDistributedGWPort() error {
 	// Only used for Error Strings
 	var nodeLocalNatSubnetNextHop string
 	if config.IPv4Mode && config.IPv6Mode {
-		dnatSnatNextHopMac = util.IPAddrToHWAddr(net.ParseIP(util.V4NodeLocalNatSubnetNextHop)).String()
-		nodeLocalNatSubnetNextHop = util.V4NodeLocalNatSubnetNextHop + " " + util.V6NodeLocalNatSubnetNextHop
+		dnatSnatNextHopMac = util.IPAddrToHWAddr(net.ParseIP(types.V4NodeLocalNATSubnetNextHop)).String()
+		nodeLocalNatSubnetNextHop = types.V4NodeLocalNATSubnetNextHop + " " + types.V6NodeLocalNATSubnetNextHop
 	} else if config.IPv4Mode {
-		dnatSnatNextHopMac = util.IPAddrToHWAddr(net.ParseIP(util.V4NodeLocalNatSubnetNextHop)).String()
-		nodeLocalNatSubnetNextHop = util.V4NodeLocalNatSubnetNextHop
+		dnatSnatNextHopMac = util.IPAddrToHWAddr(net.ParseIP(types.V4NodeLocalNATSubnetNextHop)).String()
+		nodeLocalNatSubnetNextHop = types.V4NodeLocalNATSubnetNextHop
 	} else if config.IPv6Mode {
-		dnatSnatNextHopMac = util.IPAddrToHWAddr(net.ParseIP(util.V6NodeLocalNatSubnetNextHop)).String()
-		nodeLocalNatSubnetNextHop = util.V6NodeLocalNatSubnetNextHop
+		dnatSnatNextHopMac = util.IPAddrToHWAddr(net.ParseIP(types.V6NodeLocalNATSubnetNextHop)).String()
+		nodeLocalNatSubnetNextHop = types.V6NodeLocalNATSubnetNextHop
 	}
 	stdout, stderr, err = util.RunOVNSbctl("--data=bare", "--no-heading", "--columns=_uuid", "find", "MAC_Binding",
 		"logical_port="+dgpName, fmt.Sprintf(`mac="%s"`, dnatSnatNextHopMac))
@@ -437,29 +438,29 @@ func addDistributedGWPort() error {
 	var datapath string
 	err = wait.PollImmediate(time.Second, 30*time.Second, func() (bool, error) {
 		datapath, stderr, err = util.RunOVNSbctl("--data=bare", "--no-heading", "--columns=_uuid", "find", "datapath",
-			"external_ids:name="+ovnClusterRouter)
+			"external_ids:name="+types.OVNClusterRouter)
 		// Ignore errors; can't easily detect which are transient or fatal
 		return datapath != "", nil
 	})
 	if err != nil {
 		return fmt.Errorf("failed to get the datapatah UUID of %s from OVN SB "+
-			"stdout: %q, stderr: %q, error: %v", ovnClusterRouter, datapath, stderr, err)
+			"stdout: %q, stderr: %q, error: %v", types.OVNClusterRouter, datapath, stderr, err)
 	}
 
 	if config.IPv4Mode {
-		_, stderr, err = util.RunOVNSbctl("create", "mac_binding", "datapath="+datapath, "ip="+util.V4NodeLocalNatSubnetNextHop,
+		_, stderr, err = util.RunOVNSbctl("create", "mac_binding", "datapath="+datapath, "ip="+types.V4NodeLocalNATSubnetNextHop,
 			"logical_port="+dgpName, fmt.Sprintf(`mac="%s"`, dnatSnatNextHopMac))
 		if err != nil {
 			return fmt.Errorf("failed to create a MAC_Binding entry of (%s, %s) for distributed router port %s "+
-				"stderr: %q, error: %v", util.V4NodeLocalNatSubnetNextHop, dnatSnatNextHopMac, dgpName, stderr, err)
+				"stderr: %q, error: %v", types.V4NodeLocalNATSubnetNextHop, dnatSnatNextHopMac, dgpName, stderr, err)
 		}
 	}
 	if config.IPv6Mode {
-		_, stderr, err = util.RunOVNSbctl("create", "mac_binding", "datapath="+datapath, fmt.Sprintf(`ip="%s"`, util.V6NodeLocalNatSubnetNextHop),
+		_, stderr, err = util.RunOVNSbctl("create", "mac_binding", "datapath="+datapath, fmt.Sprintf(`ip="%s"`, types.V6NodeLocalNATSubnetNextHop),
 			"logical_port="+dgpName, fmt.Sprintf(`mac="%s"`, dnatSnatNextHopMac))
 		if err != nil {
 			return fmt.Errorf("failed to create a MAC_Binding entry of (%s, %s) for distributed router port %s "+
-				"stderr: %q, error: %v", util.V6NodeLocalNatSubnetNextHop, dnatSnatNextHopMac, dgpName, stderr, err)
+				"stderr: %q, error: %v", types.V6NodeLocalNATSubnetNextHop, dnatSnatNextHopMac, dgpName, stderr, err)
 		}
 	}
 	return nil
@@ -470,35 +471,35 @@ func addPolicyBasedRoutes(nodeName, mgmtPortIP string, hostIfAddr *net.IPNet) er
 	var natSubnetNextHop string
 	if utilnet.IsIPv6(hostIfAddr.IP) {
 		l3Prefix = "ip6"
-		natSubnetNextHop = util.V6NodeLocalNatSubnetNextHop
+		natSubnetNextHop = types.V6NodeLocalNATSubnetNextHop
 	} else {
 		l3Prefix = "ip4"
-		natSubnetNextHop = util.V4NodeLocalNatSubnetNextHop
+		natSubnetNextHop = types.V4NodeLocalNATSubnetNextHop
 	}
 	// embed nodeName as comment so that it is easier to delete these rules later on.
 	// logical router policy doesn't support external_ids to stash metadata
-	matchStr := fmt.Sprintf(`inport == "rtos-%s" && %s.dst == %s /* %s */`,
-		nodeName, l3Prefix, hostIfAddr.IP.String(), nodeName)
-	_, stderr, err := util.RunOVNNbctl("lr-policy-add", ovnClusterRouter, nodeSubnetPolicyPriority, matchStr, "reroute",
+	matchStr := fmt.Sprintf(`inport == "%s%s" && %s.dst == %s /* %s */`,
+		types.RouterToSwitchPrefix, nodeName, l3Prefix, hostIfAddr.IP.String(), nodeName)
+	_, stderr, err := util.RunOVNNbctl("lr-policy-add", types.OVNClusterRouter, types.NodeSubnetPolicyPriority, matchStr, "reroute",
 		mgmtPortIP)
 	if err != nil {
 		// TODO: lr-policy-add doesn't support --may-exist, resort to this workaround for now.
 		// Have raised an issue against ovn repository (https://github.com/ovn-org/ovn/issues/49)
 		if !strings.Contains(stderr, "already existed") {
 			return fmt.Errorf("failed to add policy route '%s' for host %q on %s "+
-				"stderr: %s, error: %v", matchStr, nodeName, ovnClusterRouter, stderr, err)
+				"stderr: %s, error: %v", matchStr, nodeName, types.OVNClusterRouter, stderr, err)
 		}
 	}
 
 	// policy to allow host -> service -> hairpin back to host
 	matchStr = fmt.Sprintf("%s.src == %s && %s.dst == %s /* %s */",
 		l3Prefix, mgmtPortIP, l3Prefix, hostIfAddr.IP.String(), nodeName)
-	_, stderr, err = util.RunOVNNbctl("lr-policy-add", ovnClusterRouter, mgmtPortPolicyPriority, matchStr,
+	_, stderr, err = util.RunOVNNbctl("lr-policy-add", types.OVNClusterRouter, types.MGMTPortPolicyPriority, matchStr,
 		"reroute", natSubnetNextHop)
 	if err != nil {
 		if !strings.Contains(stderr, "already existed") {
 			return fmt.Errorf("failed to add policy route '%s' for host %q on %s "+
-				"stderr: %s, error: %v", matchStr, nodeName, ovnClusterRouter, stderr, err)
+				"stderr: %s, error: %v", matchStr, nodeName, types.OVNClusterRouter, stderr, err)
 		}
 	}
 
@@ -519,12 +520,12 @@ func addPolicyBasedRoutes(nodeName, mgmtPortIP string, hostIfAddr *net.IPNet) er
 		}
 		matchStr = fmt.Sprintf("%s.src == %s %s /* inter-%s */",
 			l3Prefix, mgmtPortIP, matchDst, nodeName)
-		_, stderr, err = util.RunOVNNbctl("lr-policy-add", ovnClusterRouter, interNodePolicyPriority, matchStr,
+		_, stderr, err = util.RunOVNNbctl("lr-policy-add", types.OVNClusterRouter, types.InterNodePolicyPriority, matchStr,
 			"reroute", natSubnetNextHop)
 		if err != nil {
 			if !strings.Contains(stderr, "already existed") {
 				return fmt.Errorf("failed to add policy route '%s' for host %q on %s "+
-					"stderr: %s, error: %v", matchStr, nodeName, ovnClusterRouter, stderr, err)
+					"stderr: %s, error: %v", matchStr, nodeName, types.OVNClusterRouter, stderr, err)
 			}
 		}
 	}
@@ -570,14 +571,10 @@ func (oc *Controller) addNodeLocalNatEntries(node *kapi.Node, mgmtPortMAC string
 		}()
 	}
 
-	mgmtPortName := util.K8sPrefix + node.Name
-	stdout, stderr, err := util.RunOVNNbctl("--if-exists", "lr-nat-del", ovnClusterRouter,
-		"dnat_and_snat", externalIP.String())
-	if err != nil {
-		return fmt.Errorf("failed to delete dnat_and_snat entry for the management port on node %s, "+
-			"stdout: %s, stderr: %q, error: %v", node.Name, stdout, stderr, err)
-	}
-	stdout, stderr, err = util.RunOVNNbctl("lr-nat-add", ovnClusterRouter, "dnat_and_snat",
+	mgmtPortName := types.K8sPrefix + node.Name
+	stdout, stderr, err := util.RunOVNNbctl("--if-exists", "lr-nat-del", types.OVNClusterRouter,
+		"dnat_and_snat", externalIP.String(), "--",
+		"lr-nat-add", types.OVNClusterRouter, "dnat_and_snat",
 		externalIP.String(), mgmtPortIfAddr.IP.String(), mgmtPortName, mgmtPortMAC)
 	if err != nil {
 		return fmt.Errorf("failed to add dnat_and_snat entry for the management port on node %s, "+
