@@ -33,7 +33,7 @@ func (oc *Controller) syncPods(pods []interface{}) {
 			continue
 		}
 		annotations, err := util.UnmarshalPodAnnotation(pod.Annotations)
-		if podScheduled(pod) && podWantsNetwork(pod) && err == nil {
+		if podScheduled(pod) && util.PodWantsNetwork(pod) && err == nil {
 			logicalPort := podLogicalPortName(pod)
 			expectedLogicalPorts[logicalPort] = true
 			if err = oc.lsManager.AllocateIPs(pod.Spec.NodeName, annotations.IPs); err != nil {
@@ -81,6 +81,23 @@ func (oc *Controller) deleteLogicalPort(pod *kapi.Pod) {
 	portInfo, err := oc.logicalPortCache.get(logicalPort)
 	if err != nil {
 		klog.Errorf(err.Error())
+		// If ovnkube-master restarts, it is also possible the Pod's logical switch port
+		// is not readded into the cache. Delete logical switch port anyway.
+		err = util.OvnNBLSPDel(oc.ovnNBClient, logicalPort)
+		if err != nil {
+			klog.Errorf(err.Error())
+		}
+
+		// Even if the port is not in the cache, IPs annotated in the Pod annotation may already be allocated,
+		// need to release them to avoid leakage.
+		logicalSwitch := pod.Spec.NodeName
+		if logicalSwitch != "" {
+			annotation, err := util.UnmarshalPodAnnotation(pod.Annotations)
+			if err == nil {
+				podIfAddrs := annotation.IPs
+				_ = oc.lsManager.ReleaseIPs(logicalSwitch, podIfAddrs)
+			}
+		}
 		return
 	}
 
@@ -90,12 +107,8 @@ func (oc *Controller) deleteLogicalPort(pod *kapi.Pod) {
 		klog.Errorf(err.Error())
 	}
 
-	cmd, err := oc.ovnNBClient.LSPDel(logicalPort)
-	if err == nil {
-		if err = oc.ovnNBClient.Execute(cmd); err != nil {
-			klog.Errorf("Error while deleting logical port: %s, %v", logicalPort, err)
-		}
-	} else if err != goovn.ErrorNotFound {
+	err = util.OvnNBLSPDel(oc.ovnNBClient, logicalPort)
+	if err != nil {
 		klog.Errorf(err.Error())
 	}
 
@@ -143,14 +156,6 @@ func (oc *Controller) addRoutesGatewayIP(pod *kapi.Pod, podAnnotation *util.PodA
 			}
 		}
 	}
-	// DUALSTACK FIXME: hybridOverlayExternalGW is not Dualstack
-	var hybridOverlayExternalGW net.IP
-	if config.HybridOverlay.Enabled {
-		hybridOverlayExternalGW, err = oc.getHybridOverlayExternalGwAnnotation(pod.Namespace)
-		if err != nil {
-			return err
-		}
-	}
 
 	for _, podIfAddr := range podAnnotation.IPs {
 		isIPv6 := utilnet.IsIPv6CIDR(podIfAddr)
@@ -158,14 +163,6 @@ func (oc *Controller) addRoutesGatewayIP(pod *kapi.Pod, podAnnotation *util.PodA
 		if err != nil {
 			return err
 		}
-		// DUALSTACK FIXME: hybridOverlayExternalGW is not Dualstack
-		// When oc.getHybridOverlayExternalGwAnnotation() supports dualstack, return error if no match.
-		// If external gateway mode is configured, need to use it for all outgoing traffic, so don't want
-		// to fall back to the default gateway here
-		if hybridOverlayExternalGW != nil && utilnet.IsIPv6(hybridOverlayExternalGW) != isIPv6 {
-			klog.Warningf("Pod %s/%s has no external gateway for %s", pod.Namespace, pod.Name, util.IPFamilyName(isIPv6))
-			continue
-		}
 
 		gatewayIPnet := util.GetNodeGatewayIfAddr(nodeSubnet)
 
@@ -176,7 +173,7 @@ func (oc *Controller) addRoutesGatewayIP(pod *kapi.Pod, podAnnotation *util.PodA
 		var gatewayIP net.IP
 		hasRoutingExternalGWs := len(oc.getRoutingExternalGWs(pod.Namespace)) > 0
 		hasPodRoutingGWs := len(oc.getRoutingPodGWs(pod.Namespace)) > 0
-		if otherDefaultRoute || (hybridOverlayExternalGW != nil && !hasRoutingExternalGWs && !hasPodRoutingGWs) {
+		if otherDefaultRoute || (hasRoutingExternalGWs && hasPodRoutingGWs) {
 			for _, clusterSubnet := range config.Default.ClusterSubnets {
 				if isIPv6 == utilnet.IsIPv6CIDR(clusterSubnet.CIDR) {
 					podAnnotation.Routes = append(podAnnotation.Routes, util.PodRoute{
@@ -193,9 +190,6 @@ func (oc *Controller) addRoutesGatewayIP(pod *kapi.Pod, podAnnotation *util.PodA
 					})
 				}
 			}
-			if hybridOverlayExternalGW != nil {
-				gatewayIP = util.GetNodeHybridOverlayIfAddr(nodeSubnet).IP
-			}
 		} else {
 			gatewayIP = gatewayIPnet.IP
 		}
@@ -238,15 +232,6 @@ func (oc *Controller) getRoutingPodGWs(ns string) map[string][]net.IP {
 	return nsInfo.routingExternalPodGWs
 }
 
-func (oc *Controller) getHybridOverlayExternalGwAnnotation(ns string) (net.IP, error) {
-	nsInfo, err := oc.waitForNamespaceLocked(ns)
-	if err != nil {
-		return nil, err
-	}
-	defer nsInfo.Unlock()
-	return nsInfo.hybridOverlayExternalGW, nil
-}
-
 func (oc *Controller) addLogicalPort(pod *kapi.Pod) (err error) {
 	// If a node does node have an assigned hostsubnet don't wait for the logical switch to appear
 	if oc.lsManager.IsNonHostSubnetSwitch(pod.Spec.NodeName) {
@@ -363,6 +348,7 @@ func (oc *Controller) addLogicalPort(pod *kapi.Pod) (err error) {
 			}
 		}
 
+		releaseIPs = true
 		var networks []*types.NetworkSelectionElement
 
 		networks, err = util.GetPodNetSelAnnotation(pod, util.DefNetworkAnnotation)
@@ -407,9 +393,9 @@ func (oc *Controller) addLogicalPort(pod *kapi.Pod) (err error) {
 		klog.V(5).Infof("Annotation values: ip=%v ; mac=%s ; gw=%s\nAnnotation=%s",
 			podIfAddrs, podMac, podAnnotation.Gateways, marshalledAnnotation)
 		if err = oc.kube.SetAnnotationsOnPod(pod, marshalledAnnotation); err != nil {
-			releaseIPs = true
 			return fmt.Errorf("failed to set annotation on pod %s: %v", pod.Name, err)
 		}
+		releaseIPs = false
 	}
 
 	// set addresses on the port
